# File tres/Scripts/trssum.cl
# October 29, 2008
# By Doug Mink, SAO Telescope Data Center

# TRSSUM - Use mscred.combine to add together a list of images

procedure trssum (images)

string	images=""	{prompt="Images to combine"}

int	sumseq=201	{prompt="Sequence number to use for output file"}
string	outroot=""	{prompt="Root of output file name"}
file	outfile=""	{prompt="Output file name (returned)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

struct *sumfiles

begin

file	templist, imout, imin1
string	tchar, imagelist, history, dstring, oroot, fn1, fn2
file	listfile, imfile
bool	dellist, vb, done
int	nfiles
real	mjdsum, mjdavg, exptot, bcvsum, bcvavg, hmjdsum, hjdavg

	vb = verbose
	dellist = deltemp
	imagelist = images
	exptot = 0.0
	fn1 = "0"
	oroot = outroot

# Load needed packages if necessary
	if (!defpac ("mscred"))
	    mscred

#  Make temporary list of input files
	templist = mktemp ("tempcombine") // ".list"
	files (imagelist, >templist)

# Count files and initialize output root name
	nfiles = 0
	done = yes
	sumfiles = templist
	mjdsum = 0.0
	bcvsum = 0.0
	hmjdsum = 0.0
	while (fscan (sumfiles,imin1) != EOF) {
	    nfiles = nfiles + 1
	    if (nfiles == 1) {
		smatch (imin1, ".fits")
		if (oroot == "") {
		    if (smatch.found) {
			oroot = substr (imin1, 6, smatch.start-1)
			}
		    else {
			oroot = substr (imin1, 6, 100)
			}
		    fn1 = substr (imin1,1,4)
		    }
		}
	    imgetp.value = 0.0
	    imgetp (imin1, "EXPTIME")
	    exptot = exptot + real (imgetp.value)
	    imgetp.value = 0.0
	    imgetp (imin1, "MJD")
	    mjdsum = mjdsum + real (imgetp.value)
	    imgetp.value = 0.0
	    imgetp (imin1, "BCV")
	    bcvsum = bcvsum + real (imgetp.value)
	    imgetp.value = 0.0
	    imgetp (imin1, "HJD")
	    hmjdsum = hmjdsum + (real (imgetp.value) + 2400000.5)
	    fn2 = substr (imin1,1,4)
	    }
	if (nfiles == 0) {
	    print ("TRSSUM: No files to process")
	    return
	    }
	else if (nfiles == 1) {
	    print ("TRSSUM: Only one file, returning it as sum")
	    imout = oroot // "_sum.fits"
	    copy (imin1, imout)
	    return
	    }

	if (fn1 == "0")
	    imout = oroot // "_sum.fits"
	else
	    imout = sumseq // "." // oroot // "_sum_" // fn1 // "_" // fn2 // ".fits"

	if (vb) {
	    print ("TRSSUM: adding ",nfiles," files from ", imagelist," to ", imout)
	    }

	combine.input = "@" // templist
	combine.output = imout
	combine.headers = ""
	combine.bpmasks = ""
	combine.rejmasks = ""
	combine.nrejmasks = ""
	combine.expmasks = ""
	combine.sigmas = ""
	combine.ccdtype = ""
	combine.amps = no
	combine.subsets = no
	combine.delete = no
	combine.combine = "sum"
	combine.reject = "none"
	combine.project = no
	combine.outtype = "real"
	combine.outlimits = ""
	combine.offsets = "none"
	combine.masktype = "none"
	combine.maskvalue = 0.
	combine.blank = 0.
	combine.scale = "none"
	combine.zero = "none"
	combine.weight = "none"
	combine.statsec = ""
	combine.lthreshold = INDEF
	combine.hthreshold = INDEF
	combine.nlow = 1
	combine.nhigh = 1
	combine.nkeep = 1
	combine.mclip = yes
	combine.lsigma = 3.
	combine.hsigma = 3.
	combine.rdnoise = "0."
	combine.gain = "1."
	combine.snoise = "0."
	combine.sigscale = 0.1
	combine.pclip = -0.5
	combine.grow = 0.

	combine (imagelist, imout)

# Convert time of exposure to mean of added files
	mjdavg = mjdsum / real (nfiles)
	round (mjdavg, "%10.5f")
	hedit (imout, "MJD",round.value,update+,verify-,show=vb)
	gettime (mjdavg, usesec+)
	hedit (imout, "DATE-OBS",gettime.fitsdate,update+,verify-,show=vb)

# Set barycentric velocity correction to mean of those in added files
	bcvavg = bcvsum / real (nfiles)
	round (bcvavg, "%10.5f")
	hedit (imout, "BCV",round.value,update+,verify-,show=vb)

# Set heliocentric Julian Date to mean of those in added files
	hjdavg = (hmjdsum / real (nfiles)) + 2400000.5
	round (hjdavg, "%10.5f")
	hedit (imout, "HJD",round.value,update+,verify-,show=vb)

# Update exposure time to total of those summed
	round (exptot, "%.1f")
	hedit (imout, "EXPTIME", round.value, update+, verify-,show=vb)

# Make history line including time
	gettime (0.0)
	dstring = gettime.date // " " // gettime.time
	history = dstring // " " // nfiles // " images summed"
	hedit (imout, "TRSSUM", history, add+, verify-)
	outfile = imout

# Increment sequence number for this directory
	sumseq = sumseq + 1

	if (dellist) {
	    if (access (templist))
		delete (templist)
	    }
	sumfiles = ""

end

# Dec 13 2007	New program for TRES pipeline

# Jan 10 2008	Change name from tressum to trssum
# Jan 31 2008	Add TRSSUM keyword to output header
# Apr 10 2008	Create output file name from input list and sequence number
# May 16 2008	Set DATE-OBS and MJD to the mean observation time
# May 16 2008	Set EXPTIME to the total exposure duration
# May 16 2008	Add input time argument to gettime() 
# Jul 17 2008	Add outroot parameter to specify filename
# Jul 31 2008	Drop unused parameter saveold
# Sep 22 2008	Clean up code
# Oct 29 2008	Update HJD and BCV as well as MJD and EXPTIME
# Oct 29 2008	Use separate list structure fore safety in scripts
