# File tres/Scripts/trsdate.cl
# By Doug Mink
# October 14, 2008

# TRSDATE -- Set instrument and date directories for TRES listing and quick look

procedure trsdate (datedir)

string	datedir="now"	{prompt="Date for images (now=today)"}

string	instdir="/tdata/tres/" {prompt="Root directory with raw data"}
string	quickdir="home$tresq/"	{prompt="Root directory for quicklook data"}
int	lastseq=200	{prompt="First sequence number for summed spectra"}
bool	deltemp=yes	{prompt="Delete temporary list file (yes or no)"}
bool	verbose=yes	{prompt="Print table of image parameters (yes or no)"}

struct *dirfiles

begin
    bool vb
    int ismax, iseq
    file ddir, idir, datadir, qdir, templist, imin
    string rawfiles

    vb = verbose
    ddir = datedir

    fixdir (instdir)
    idir = fixdir.newdir
    if (!access (idir)) {
	print ("TRSDATE: Raw data directory ",idir, " not found")
	return
	}

    fixdir (quickdir)
    qdir = fixdir.newdir

    if (ddir == "now" || ddir == "" || ddir == "0") {
	gettime (0.0)
	ddir = gettime.datadir
	}
    if (ddir == "now-1" || ddir == "1") {
	gettime (0.0, datediff=-1)
	ddir = gettime.datadir
	}

    datadir = idir // ddir
    if (!access (datadir)) {
	print ("TRSDATE: Raw data directory ",datadir, " not found")
	return
	}

    qtres.datedir = ddir
    qtres.instdir = idir
    ctres.datedir = ddir
    ctres.instdir = idir
    trsproc.datedir = ddir
    trsproc.instdir = idir
    ftres.datedir = ddir
    ftres.instdir = idir
    ttres.datedir = ddir
    ttres.instdir = idir
    trslist.datedir = ddir
    trslist.instdir = idir
    trsgroup.datedir = ddir
    trsgroup.instdir = idir
    trsfiles.datedir = ddir
    trsfiles.instdir = idir
    trsfiles.adddate = no


    if (!access (qdir)) {
	mkdir (qdir)
	if (!access (qdir)) {
	    print ("TRSDATE: ",qdir, " cannot be created")
	    return
	    }
	}
    qdir = qdir // ddir
    if (!access (qdir)) {
	mkdir (qdir)
	if (!access (qdir)) {
	    print ("TRSDATE: ",qdir, " cannot be created")
	    return
	    }
	}
    cd (qdir)

    ismax = lastseq
    rawfiles = idir // ddir // "/0*.fits"
    templist = mktemp ("tempfile") // ".list"
    files (rawfiles, >templist)
    dirfiles = templist
    while (fscan (dirfiles, imin) != EOF) {
	filename (imin)
	iseq = int (substr (filename.filename,1,4))
	if (iseq > ismax)
	    ismax = iseq
	}

    trssum.sumseq = ismax + 1

    if (vb) {
	print ("Read from ",datadir)
	print ("Write to ",qdir)
	print ("Current working directory is ",qdir)
	}

    if (deltemp) {
	delete (templist)
	dirfiles = ""
	}
end

# Feb 13 2008	New program
# Feb 15 2008	Add trsfiles
# Apr  4 2008	First sequence number for summed spectra on this night
# Apr  4 2008	Move to local date directory, creating it if necessary
# Apr  4 2008	Set trsfiles.adddate=no because date is not needed in name
# Apr 22 2008	Add quicklook primary directory and create it if needed
# May 16 2008	Add input time argument to gettime() 
# Jul 14 2008	Add TRSGROUP
# Jul 16 2008	Add TRSPROC, FTRES, and TTRES
# Jul 31 2008	Set next sequence number for sums and averages in this directory
# Aug  1 2008	Skip files which start with "flat" or "comp"
# Aug 14 2008	List files correctly and delete temporary list file
# Oct 14 2008	Print input and output directories on separate lines
# Oct 14 2008	Change default raw data directory to /tdata/tres/
