# File tres/Scripts/trslist.cl
# By Doug Mink
# July 15, 2008

# TRSLIST -- Make Starbase table of major keywords in TRES data files
#            and make lists for FLATs and COMPs

procedure trslist (images)

string images="1-300"	{prompt="Files to process"}

string	datedir=""	{prompt="Date for images"}
file	instdir=""	{prompt="Directory containing date directories"}
file	imagedir=""	{prompt="Directory containing images (overrides above)"}
string	imagetype=""	{prompt="Select this image type if not null"}
string	object=""	{prompt="Select this object if not null"}
string	apfib=""	{prompt="Select fiber (s m l) binning (b) ap (1 2 3)"}
string	aperture=""	{prompt="Select on this aperture (1 2 3)"}
string	fibkey=""	{prompt="Select on this FIBKEY value"}
string	binning=""	{prompt="Select if binned this many times (1 2 null)"}
string	fibsize=""	{prompt="Select if FIBSIZ is this value"}
string	exptime=""	{prompt="Select if EXPTIME has this value"}
file	outlist=""	{prompt="File name for output list of files"}
bool	addfib=yes	{prompt="Add FIBi keywords to spectra (yes or no)"}
bool	deltemp=yes	{prompt="Delete temporary files (yes or no)"}
bool	overwrite=no	{prompt="Overwrite existing list files (yes or no)"}
bool	verbose=yes	{prompt="Print table of image parameters (yes or no)"}
bool	debug=no	{prompt="Print table with more image parameters (yes or no)"}

struct *filelist

begin
    bool vb, db, dt, ow
    file imf, tempin, tempout, idir, imout
    file listfile, ofile, impath, imin
    string fkey, fib1, fib2, fib3, af, aff, afb, afs, afx
    string bin1, ims, ddir, fk, fsize, imt, ut
    int ns, n, nrange, lfile, maxlfile, lobj, maxlobj
    string specs, olist, lastpath, obj
    string fname, fpath
    string dline1, dline2
    char tchar
    int iexp, iexp0
    real mjdmid

    vb = verbose
    db = debug
    ow = overwrite
    dt = deltemp
    ims = images
    if (imagedir == "") {
	idir = instdir
	ddir = datedir
	}
    else {
	idir = imagedir
	ddir = ""
	}
    ofile = outlist
    fk = fibkey
    afs = fibsize
    afb = binning
    aff = aperture
    imt = imagetype
    if (exptime != "")
	iexp = int (exptime)
    else
	iexp = 0

    if (db)
	print ("TRSLIST: Finding fiber size")

# Find fiber size, binning, and 
    af = apfib
    afx = af
    if (afx != "") {
	trsdecode (afx)
	afs = trsdecode.fibsize
	afb = trsdecode.binning
	aff = trsdecode.apertures
	}

    maxlfile = 0
    maxlobj = 0

    if (idir != "" || isrange.range)
	addfib = no

# Make list of image paths
    trsfiles.instdir = idir
    trsfiles.datedir = ddir
    trsfiles.copyfile = no
    trsfiles.verbose = no
    trsfiles.debug = no
    if (db) {
	print ("TRSLIST: About to call TRSFILES apsize=", afs, " binning=",afb, " fiber=",aff)
	print ("TRSLIST: Reading from ",ims
	}
    trsfiles (ims)
    tempin = trsfiles.tempin
    tempout = trsfiles.tempout
    ims = "@" // tempout
    if (db)
	print ("TRSLIST: File list in ",tempout)

    maxlfile = trsfiles.maxlfile
    if (maxlfile < 8)
	maxlfile = 8
    maxlobj = trsfiles.maxlobj
    if (maxlobj < 6)
	maxlobj = 6

# Print heading for list of spectrum attributes
    if (debug) {
	fname = "filename"
	lfile = strlen (fname)
	for (i = lfile+1; i <= maxlfile; i = i + 1) {
	    fname = fname // " "
	    }
	obj = "object"
	for (i = 7; i <= maxlobj; i = i + 1) {
	    obj = obj // " "
	    }
	dline1 = fname // "    	type	" // obj // "	peri	bin	fibsize	ossw	exptime	cmptime	fibkey	fib1	fib2	fib3"
	fname = "-"
	for (i = 2; i <= maxlfile; i = i + 1) {
	    fname = fname // "-"
	    }
	obj = "-"
	for (i = 2; i <= maxlobj; i = i + 1) {
	    obj = obj // "-"
	    }
	dline2 = fname // "	-------	" // obj // "	----	---	-------	----	-------	-------	------	----	----	----"
	}
    else {
	fname = "filename"
	for (i = 9; i <= maxlfile; i = i + 1) {
	    fname = fname // " "
	    }
	obj = "object"
	for (i = 7; i <= maxlobj; i = i + 1) {
	    obj = obj // " "
	    }
	dline1 = fname // "	type	"// obj // "	mjdmid     	apfib	fibkey	exptime	cmptime"
	fname = "-"
	for (i = 2; i <= maxlfile; i = i + 1) {
	    fname = fname // "-"
	    }
	obj = "-"
	for (i = 2; i <= maxlobj; i = i + 1) {
	    obj = obj // "-"
	    }
	dline2 = fname // "	-------	" // obj // "	-----------	----	-------	-------	-------"
	}
    
    tfib1.addfib = addfib
    tfib1.verbose = no
    tfib1.debug = no

    ns = 0

# Print spectrum attributes
    olist = ""
    filelist = tempout
    while (fscan (filelist, imf) != EOF) {
	fpath = imf
	filename (fpath)
	fname = filename.filename
	tfib1.specdir = filename.dirpath
	tfib1 (fname)
#	if (db)
#	    dpar ("tfib1")
	fkey = tfib1.fibkey
	fib1 = tfib1.fib1
	fib2 = tfib1.fib2
	fib3 = tfib1.fib3
	fsize = tfib1.fibsize
	bin1 = tfib1.bin1
	ut = substr (tfib1.dateobs,12,19)
	mjdmid = tfib1.mjdmid
#	if (db)
#	    print ("AF=",af," AFF=",aff," afs=",afs," afb=",afb)
#	if (db)
#	    print ("AF=",af," AFF=",aff," afs=",afs,"/",tfib1.fibsize," afb=",afb,"/",bin1)
	if (af != "") {
	    if (aff == "1" && fib1 == "OFF")
		next
	    if (aff == "2" && fib2 == "OFF")
		next
	    if (aff == "3" && fib3 == "OFF")
		next
	    if (afb != bin1)
		next
	    if (afs != "" && afs != tfib1.fibsize)
		next
	    }
	if (imt != "" && tfib1.imagetype != imt) {
	    if (db)
		print ("TRSLIST: image type ",imt," != ", tfib1.imagetype)
	    next
	    }
	if (fk != "" && fkey != fk) {
	    if (db)
		print ("TRSLIST: key ",fk," != ", fkey)
	    next
	    }
	if (afs != "" && fsize != afs) {
	    if (db)
		print ("TRSLIST: fiber size ",afs," != ", fsize)
	    next
	    }
	if (afb != "" && bin1 != afb) {
	    if (db)
		print ("TRSLIST: binning ",afb," != ", bin1)
	    next
	    }
	if (object != "" && tfib1.object != object) {
	    if (db)
		print ("TRSLIST: object ",tfib1.object," != ", object)
	    next
	    }
	if (iexp > 0) {
	    iexp0 = int (tfib1.exptime)
	    if (iexp0 != iexp) {
		if (debug)
		    print ("TRSLIST: ",tfib1.exptime," != ",exptime)
		next
		}
	    }
	ns = ns + 1
	if (vb && ns == 1) {
#	    print (idir)
	    print (dline1)
	    print (dline2)
	    }
	lfile = strlen (fname)
	for (i = lfile+1; i <= maxlfile; i = i + 1) {
	    fname = fname // " "
	    }
	obj = tfib1.object
	lobj = strlen (obj)
	for (i = lobj+1; i <= maxlobj; i = i + 1) {
	    obj = obj // " "
	    }

	if (db)
	    printf ("%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s\n", fname, tfib1.imagetype, tfib1.object, tfib1.peri, bin1, fsize, tfib1.osswitch, tfib1.exptime, tfib1.comptime, fkey, fib1, fib2, fib3)
	else if (vb)
	    printf ("%s	%s	%s	%s	%s	%s	%s	%s\n", fname, tfib1.imagetype, obj, ut, tfib1.apfib, fkey, tfib1.exptime, tfib1.comptime)

# Save this file name in list or listfile
	if (ns == 1) {
	    if (access (ofile) && ow)
		delete (ofile)
	    if (ofile != "") {
		if (vb)
		    printf ("%s	%s	%s	%11.5f	%s	%s	%s	%s\n", fname, tfib1.imagetype, obj, mjdmid, tfib1.apfib, fkey, tfib1.exptime, tfib1.comptime, > ofile)
		else
		    print (fname, > ofile)
		}
	    else
		olist = fname
	    }
	else {
	    if (ofile != "") {
		if (vb)
		    printf ("%s	%s	%s	%11.5f	%s	%s	%s	%s\n", fname, tfib1.imagetype, obj, mjdmid, tfib1.apfib, fkey, tfib1.exptime, tfib1.comptime, >> ofile)
		else
		    print (fname, >> ofile)
		}
	    else
		olist = olist // "," // fname
	    }
	}

    if (dt) {
	if (access (tempin))
	    delete (tempin)
	if (access (tempout))
	    delete (tempout)
	}

    if (!vb && !db && olist != "")
	print (olist)
    filelist = ""

end

# Oct  1 2007	New program
# Oct  5 2007	Use tresfib to set FIBi keyword values
# Oct  9 2007	Use header parameter values from tresfib, not gethead
# Oct 10 2007	If speckey.db already exists, print it if verbose=yes
# Oct 11 2007	Add apfib and object keywords to use treslist for sorting
# Oct 11 2007	If not verbose and conditions set, output a list of files
# Oct 11 2007	Set speckey.db in tresdb.cl, not here
# Oct 12 2007	Update parameter list
# Oct 25 2007	Add output file for long lists meeting apfib/object condition
# Dec 19 2007	Add parameter fibkey and test for equality to it if not null

# Jan 10 2008	Change name from treslist to trslist
# Jan 10 2008	Changes calls to tresfib to trsfib
# Jan 28 2008	Add optional check for a specific exposure time
# Jan 30 2008	Add imagetype to list only spectra from a specific object type
# Jan 30 2008	Parse observation sequence numbers from arbitrary directory
# Jan 31 2008	Shrink the verbose output table,leaving a bigger one for debug
# Feb 12 2008	Change parameter datedir to imdate for constency with QTRES and CTRES
# Feb 13 2008	Change parameter imdate back to datedir because it makes more sense
# Feb 15 2008	Parse input image file range/list using TRSFILES
# Mar 19 2008	Specify filelist as input list
# May  6 2008	Print heading only after a file is found
# May  6 2008	Fix bug so image directory is printed above the heading
# May 16 2008	Add overwrite parameter to overwrite a pre-existing list file
# May 19 2008	Parse apfib parameter using tdecomp() task
# Jun  6 2008	Call TRSFIB1 instead of TRSFIB
# Jul 11 2008	Use aff, afb, and afs instead of fb and fs
# Jul 11 2008	If verbose, print table to outlist; if not, print filenames
# Jul 15 2008	If verbose, print MJD midtime instead of UT for time matching
