# File tres/Scripts/trsmed.cl
# October 29, 2008
# By Doug Mink, SAO Telescope Data Center

# TRSMED - Use mscred.combine to compute the median of a list of images

procedure trsmed (images)

string	images=""	{prompt="Images to combine"}

string  outroot=""      {prompt="Root of output file name"}
file	outfile=""	{prompt="Output file name (returned)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

struct *medfiles

begin

file	templist, imout, imin1
int	medseq, nfiles
string	tchar, imagelist, history, dstring, oroot, fn1, fn2
file	listfile, imfile
bool	dellist, vb
real	mjdsum, mjdavg, exptot, bcvsum, bcvavg, hmjdsum, hjdavg
real	exp, mjd, utesum, uteavg, airsum, haavg, stavg, airavg, expavg

	vb = verbose
	dellist = deltemp
	imagelist = images
	imout = outfile
	fn1 = "0"
	medseq = trssum.sumseq

# Load needed packages if necessary
	if (!defpac ("mscred"))
	    mscred

#  Make temporary list of input files
	templist = mktemp ("tempmed") // ".list"
	files (imagelist, >templist)

# Count files and initialize output root name
	nfiles = 0
	done = yes
	medfiles = templist
	mjdsum = 0.0
	bcvsum = 0.0
	hmjdsum = 0.0
	utesum = 0.0
	airsum = 0.0
	hasum = 0.0
	stsum = 0.0
	while (fscan (medfiles,imin1) != EOF) {
	    nfiles = nfiles + 1
	    if (nfiles == 1) {
		smatch (imin1, ".fits")
		if (oroot == "") {
		    if (smatch.found) {
			oroot = substr (imin1, 6, smatch.start-1)
			}
		    else {
			oroot = substr (imin1, 6, 100)
			}
		    fn1 = substr (imin1,1,4)
		    }
		}
	    imgetp.value = 0.0
	    imgetp (imin1, "EXPTIME")
	    exp = real (imgetp.value)
	    exptot = exptot + exp
	    imgetp.value = 0.0
	    imgetp (imin1, "MJD")
	    mjd = real (imgetp.value)
	    mjdsum = mjdsum + mjd
	    utesum = utesum + (mjd + (exp / 86400.0))
	    imgetp.value = 0.0
	    imgetp (imin1, "ST")
	    stsum = stsum + real (imgetp.value)
	    imgetp.value = 0.0
	    imgetp (imin1, "HA")
	    hasum = hasum + real (imgetp.value)
	    imgetp.value = 0.0
	    imgetp (imin1, "AIR")
	    airsum = airsum + real (imgetp.value)
	    imgetp.value = 0.0
	    imgetp (imin1, "BCV")
	    bcvsum = bcvsum + real (imgetp.value)
	    imgetp.value = 0.0
	    imgetp (imin1, "HJDN")
	    hmjdsum = hmjdsum + (real (imgetp.value) + 2400000.5)
	    fn2 = substr (imin1,1,4)
	    }
	if (nfiles == 0) {
	    print ("TRSMED: No files to process")
	    return
	    }
	else if (nfiles == 1) {
	    print ("TRSMED: Only one file, returning it as median file")
	    imout = oroot // "_sum.fits"
	    copy (imin1, imout)
	    return
	    }

	if (fn1 == "0")
	    imout = oroot // "_med.fits"
	else
	    imout = medseq // "." // oroot // "_med_" // fn1 // "_" // fn2 // ".fits"

	if (vb) {
	    print ("TRSMED: median of ",nfiles," files from ", imagelist," to ", imout)
	    }

	combine.input = "@" // templist
	combine.output = imout
	combine.headers = ""
	combine.bpmasks = ""
	combine.rejmasks = ""
	combine.nrejmasks = ""
	combine.expmasks = ""
	combine.sigmas = ""
	combine.ccdtype = ""
	combine.amps = no
	combine.subsets = no
	combine.delete = no
	combine.combine = "median"
	combine.reject = "ccdclip"
	combine.project = no
	combine.outtype = "real"
	combine.outlimits = ""
	combine.offsets = "none"
	combine.masktype = "none"
	combine.maskvalue = 0.
	combine.blank = 0.
	combine.scale = "exposure"
	combine.zero = "none"
	combine.weight = "none"
	combine.statsec = ""
	combine.lthreshold = INDEF
	combine.hthreshold = INDEF
	combine.nlow = 1
	combine.nhigh = 1
	combine.nkeep = 1
	combine.mclip = yes
	combine.lsigma = 3.
	combine.hsigma = 3.
	combine.rdnoise = "0."
	combine.gain = "1."
	combine.snoise = "0."
	combine.sigscale = 0.1
	combine.pclip = -0.5
	combine.grow = 0.

	combine (imagelist, imout)

# Convert time of exposure to mean of combined files
	mjdavg = mjdsum / real (nfiles)
	round (mjdavg, "%10.5f")
	hedit (imout, "MJD",round.value,update+,verify-,show=vb)
	gettime (mjdavg, usesec+)
	hedit (imout, "DATE-OBS",gettime.fitsdate,update+,verify-,show=vb)
	hedit (imout, "UTOPEN", gettime.time, update+, verify-, show=vb)
	uteavg = utesum / real (nfiles)
	gettime (uteavg, usesec+)
	hedit (imout, "UTEND", gettime.time, update+, verify-, show=vb)

# Convert hour angle to mean of input files
	haavg = hasum / real (nfiles)
	gettime (haavg, usesec+)
	hedit (imout, "HA",gettime.time,update+,verify-,show=vb)

# Convert sidereal time to mean of input files
	stavg = stsum / real (nfiles)
	gettime (stavg, usesec+)
	hedit (imout, "HA",gettime.time,update+,verify-,show=vb)

# Convert airmass to mean of input files
	airavg = airsum / real (nfiles)
	round (airavg, "%6.3f")
	hedit (imout, "AIR",round.value,update+,verify-,show=vb)

# Set barycentric velocity correction to mean of those in combined files
	bcvavg = bcvsum / real (nfiles)
	round (bcvavg, "%10.5f")
	hedit (imout, "BCV",round.value,update+,verify-,show=vb)

# Set heliocentric Julian Date to mean of those in combined files
	hjdavg = (hmjdsum / real (nfiles)) + 2400000.5
	round (hjdavg, "%10.5f")
	hedit (imout, "HJDN",round.value,update+,verify-,show=vb)

# Update exposure time to mean of those in combined files
	expavg = exptot / real (nfiles)
	round (exptot, "%.1f")
	hedit (imout, "EXPTIME", round.value, update+, verify-,show=vb)

# Make history line including time
	gettime (0.0)
	dstring = gettime.date // " " // gettime.time
	history = dstring // " median of " // nfiles // " images"
	hedit (imout, "TRSMED", history, add+, verify-)

# Increment sequence number for this directory
	trssum.sumseq = trssum.sumseq + 1

	if (dellist) {
	    if (access (templist))
		delete (templist)
	    }
	medfiles = ""
end

# Dec 14 2007	New program for TRES pipeline

# Jan 10 2008	Change name from tresmed to trsmed
# Jan 31 2008	Add TRSMED keyword to output header
# Feb 11 2008	Change temporary file from tempcombine*list to tempmed*.list
# May 16 2008	Add input time argument to gettime() 
# Oct 29 2008	Modify MJD, DATE-OBS, BCV, HJDN, EXPTIME keywords
