# File tres/Scripts/tsetbcv.cl
# By Doug Mink, SAO Telescope Data Center
# October 29, 2008

# TSETBCV -- Add barycentric velocity correction (BCV) to image header

procedure tsetbcv (images)

string	images = ""	{prompt="Files for which to set barycentric velocity correction"}

bool	deltemp = no	{prompt="Delete temporary file (y or n)"}
bool	verbose = no	{prompt="Display heliocentric and barycentric correction (yes or no)"}
bool	debug = no	{prompt="Show intermediate computed values (yes or no)"}

struct *bcvfiles

begin
    file imin		# File for which to set barycentric velocity correction
    string ims
    file templist
    string imtype
    bool vb, db, dt

    ims = images
    vb = verbose
    db = debug
    dt = deltemp

    if (!defpac ("rvsao"))
	rvsao (motd=no)

#  Make temporary list of input files
    templist = mktemp ("tempbcvp") // ".list"
    files (ims, >templist)

    bcvfiles = templist
    while (fscan (bcvfiles,imin) != EOF) {

	imgetp.default = ""
	imgetp (imin, "IMAGETYP")
	imtype = imgetp.value
	if (imtype == "OBJECT") {
	    bcvcorr.spectra = ""
	    bcvcorr.specnum = "0"
	    bcvcorr.specdir = ""
	    bcvcorr.specsky = no
	    bcvcorr.keyra = "RA"
	    bcvcorr.keydec = "DEC"
	    bcvcorr.keyeqnx = "EPOCH"
	    bcvcorr.ra = "INDEF"
	    bcvcorr.dec = "INDEF"
	    bcvcorr.equinox = "INDEF"
	    bcvcorr.keydate = "DATE-OBS"
	    bcvcorr.keytime = "UTOPEN"
	    bcvcorr.keywhen = "start"
	    bcvcorr.keyexp = "EXPTIME"
	    bcvcorr.keyhjd = ""
	    bcvcorr.hjd = 0.
	    bcvcorr.gjd = 0.
	    bcvcorr.year = INDEF
	    bcvcorr.month = INDEF
	    bcvcorr.day = INDEF
	    bcvcorr.ut = INDEF
	    bcvcorr.obsname = ""
	    bcvcorr.keyobs = "SITENAME"
	    bcvcorr.keylat = "SITELAT"
	    bcvcorr.keylong = "SITELONG"
	    bcvcorr.keyalt = "SITEELEV"
	    bcvcorr.obslong = INDEF
	    bcvcorr.obslat = INDEF
	    bcvcorr.obsalt = INDEF
	    bcvcorr.savebcv = yes
	    bcvcorr.savejd = yes
	    bcvcorr.bcv = INDEF
	    bcvcorr.hcv = INDEF
	    bcvcorr.verbose = vb
	    bcvcorr.printmode = 1
	    bcvcorr.debug = db

# Check object name for twilight sky
	    imgetp.default = ""
	    imgetp (imin, "OBJECT")

# If sky, use earth velocity relative to the sun
	    if (imgetp.value == "SKY" || imgetp.value == "sky" || imgetp.value == "Sky")
		bcvcorr (imin,specsky=yes)

# If not sky, use vector of earth velocity relative to the sun toward object
	    else
		bcvcorr (imin,specsky=no)
	    }
	}

    if (dt)
	delete (templist)

    bcvfiles = ""
    return
end
# Oct 27 2008	New program
# Oct 29 2008	Load rvsao if it is not already loaded
