# File tres/Scripts/ttres.cl
# October 16, 2008
# By Doug Mink, SAO Telescope Data Center

# TTRES - Process multiple similar TRES ThAr images from raw data files to spectra

procedure ttres (images)

string	images=""	{prompt="Images to process"}
string	datedir=""	{prompt="Date for images"}
string	instdir=""	{prompt="Directory containing date directory"}
bool	fixpreamp=no	{prompt="Run tpreamp to fix time constant (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	flatten=yes	{prompt="Run TFLAT to Flatten image (yes or no)"}
string	normfile=""	{prompt="Pixel to pixel tmakeflat normalization file"}
string	normdir="tresdata$"	{prompt="Directory for normalization file"}
real	threshold=10.0	{prompt="Cosmic ray threshold in sigma"}
char	sumspec="no"	{prompt="Process summed 2-D file after tcosmic yes no only"}
string	aproot="flat"	{prompt="Root file name for aperture reference file"}
file	apdir="tresdata$" {prompt="Directory for aperture reference file"}
bool	recenter=no	{prompt="Recenter orders during extraction (yes or no)"}
bool	extract=yes	{prompt="Extract orders to echelle file (yes or no)"}
bool	disperse=yes	{prompt="Dispersion correct extracted spectra (yes or no)"}
bool	medshift=no	{prompt="yes: median shift no: mean shift over orders"}
file	compstd=""	{prompt="Name of file in list to use as shift standard"}
string	comproot="comp"	{prompt="Root file name for calibration lamp file"}
file	compdir="tresdata$"	{prompt="Directory for calibration lamp file"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (y or n)"}
bool	saveold=no	{prompt="Save intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, imin, impath
file	imout, ims, cdir, tempin, tempout
string	ddir, idir, croot
int	pl, n
bool	vb, dt, sr, sv, disp, rc, sum, sumonly
real	ct

	vb = verbose
	dt = deltemp
	sv = saveold
	sr = saveraw
	rc = recenter
	ddir = datedir
	idir = instdir
	ims = images
	cdir = compdir
	croot = comproot
	disp = disperse
	ct = threshold

	if (ims == "") {
	    print ("TTRES: Reduce multiple similar TRES ThAr spectra, removing cosmic rays")
	    print ("Usage: ttres images with both amps to be corrected")
	    return
	    }

	tcheck
	if (tcheck.rawdir)
	    trsdate (tcheck.datedir)

#  Make list file from input file parameter
	trsfiles.instdir = idir
	trsfiles.datedir = ddir
	trsfiles.copyfile = yes
	trsfiles.overwrite = no
	trsfiles (ims)
	tempin = trsfiles.tempin
	tempout = trsfiles.tempout
	ims = "@" // tempout

# Remove overscan, trim, and merge amplifiers of image
# Then remove pixel to pixel variation from image (including IR fringing)
# Then extract spectra from image and add dispersion function
	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpix = fixpix
	tproc.fixpreamp = fixpreamp
	tproc.flatten = flatten
	tproc.cosmic = yes
	tproc.sumspec = sumspec
	tproc.threshold = ct
	tproc.extract = extract
	tproc.disperse = disp
	tproc.normfile = normfile
	tproc.normdir = normdir
	tproc.aproot = aproot
	tproc.apdir = apdir
	tproc.medshift = medshift
	tproc.refroot = croot
	tproc.refdir = cdir
	tproc.recenter = rc
	tproc.deltemp = dt
	tproc.saveraw = sr
	tproc.saveold = sv
	tproc.verbose = vb
	tcosmic.savebp = sv
	tproc (ims, compstd=compstd)

#  Delete temporary list files
	if (dt) {
	    delete (tempin)
	    delete (tempout)
	    }

	if (tcheck.rawdir)
	    tdata (datedir=tcheck.datedir)

end

# Jul 17 2008	New task for processing ThAr spectra based on CTRES
# Sep 19 2008	Add medshift parameter which gets passed through TPROC to TCAL1
# Oct 14 2008	Do not run if in raw data directory
# Oct 14 2008	Make default to save raw files
# Oct 15 2008	Add threshold parameter to set cosmic ray threshold
# Oct 16 2008	Add compstd parameter for file name to become comp*ec.fits
# Oct 16 2008	From raw directory, switch to quick-look directory and back
